/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _ECOUNTERS_EXT_HPP_
#define _ECOUNTERS_EXT_HPP_

#include "tlm.h"

struct PwtTargetModule;

class side_effect_extension: public tlm::tlm_extension<side_effect_extension> {
public:
  side_effect_extension(std::vector<PwtTargetModule*> *se): m_side_effects(se) {}
  
  inline void add_side_effect(PwtTargetModule *se) {
    assert(se && "cannot add NULL side effect instance");
    m_side_effects->push_back(se);
  }
  
  inline tlm_extension_base* clone() const {
    return new side_effect_extension(m_side_effects);
  }
  
  inline void copy_from(tlm_extension_base const &ext) {
    const side_effect_extension* se_ext =
      dynamic_cast<const side_effect_extension*>(&ext);
    assert(se_ext!=NULL && "wrong extension type");
    m_side_effects = se_ext->m_side_effects;
  }
  
protected:
  std::vector<PwtTargetModule*> *m_side_effects;
};

#endif // _ECOUNTERS_EXT_HPP_
