//
// SimSoC Initial software, Copyright © INRIA 2007, 2008, 2009, 2010, 2011
// LGPL license version 3
//

#ifndef ELF_LOADER_HPP
#define ELF_LOADER_HPP

#include <tlm_utils/multi_passthrough_initiator_socket.h>

#include "elf.h"

#include <fstream>
#include <vector>
#include <map>
#include <stdint.h>

struct ProcessorIF;

namespace simsoc {

  struct Elf32_SectionHeader;

  struct Elf32_Header: public Elf32_Ehdr {
    std::vector<Elf32_SectionHeader*> sections;
    char * strings;
    Elf32_SectionHeader* reginfo;
    Elf32_Header();
    ~Elf32_Header();
    bool is_elf();
    bool is_elf32();
    bool is_big_endian();
    void unencode();
    bool is_exec();
    void load_sections(std::ifstream &ifs);
    size_t start_brk;
    uint32_t get_start_brk();
  };

  class ElfFile{
  public:

    const char* file_name;

    ElfFile(const char* elf_file);

    ~ElfFile();

    // Architecture get_architecture();

    // MIPS_SpecFlag get_mips_spec();

    bool is_big_endian();

    bool is_elf32() {return header.is_elf32();}

    uint32_t get_start_address();

    void load_sections(char *mem, const size_t max);

    uint32_t get_elf_brk();

    std::ifstream ifs;

    Elf32_Header header;

    bool loaded;

    size_t start_brk;
  };

  class ElfLoader: public sc_core::sc_module {
  public:

    tlm_utils::multi_passthrough_initiator_socket<ElfLoader, 32> initiator_sockets;

    ElfLoader(sc_core::sc_module_name name);
    void set_file(ProcessorIF *proc, const std::string &elf_file);
    void end_of_elaboration();

  protected:
    void load_sections(ElfFile &elf_file, int id);

    struct Job {
      ProcessorIF *proc;
      std::string filename;
      int socket_id;
      Job(ProcessorIF *p, const std::string &f, int s):
        proc(p), filename(f), socket_id(s) {}
    };
    std::vector<Job> jobs;
  };

} // namespace simsoc

#endif //ELF_LOADER_HPP
