/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _INTC_HPP_
#define _INTC_HPP_

#include "basic.hpp"
#include "pwt_module.hpp"

class Intc: public sc_core::sc_module,
            public PwtModule {
public:
  
  basic::target_socket<Intc> target;

  sc_core::sc_in<bool> **in_ports;

  sc_core::sc_out<bool> out;

  SC_HAS_PROCESS(Intc);
  Intc(sc_core::sc_module_name, unsigned nb_in);
  ~Intc();

  tlm::tlm_response_status
  read(uint32_t addr, uint32_t& data);
  
  tlm::tlm_response_status
  write(uint32_t addr, uint32_t data);

protected:
  const unsigned m_nb_in; // number of in ports

  struct EventCatcher: sc_core::sc_module {
    SC_HAS_PROCESS(EventCatcher);
    EventCatcher(sc_core::sc_module_name,
                 Intc *parent,
                 unsigned id);
    Intc &m_parent;
    const unsigned m_id;
    void process_in_irq();
  };
  EventCatcher **event_catchers;

  void process_in_irq(unsigned id);

  void send_irq();
  void send_irq_maybe(unsigned mask);

  // Masked or not
  uint32_t m_enabled_it;
  // Received and not ACK-ed
  uint32_t m_active_it;
  
  bool irq_to_send_notified;
  sc_core::sc_event irq_to_send;
};

#endif // _INTC_HPP_
