/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _MEMORY_HPP_
#define _MEMORY_HPP_

#include <tlm_utils/simple_target_socket.h>
#include "pwt_module.hpp"
#include "ecounters_ext.hpp"

class Memory: public sc_core::sc_module,
              public PwtTargetModule
{
public:

  static const unsigned READ_LATENCY = 1;
  static const unsigned WRITE_LATENCY = 1;

  tlm_utils::simple_target_socket<Memory> target;

  Memory(sc_core::sc_module_name name, unsigned int byte_size);

  ~Memory();

  // RW_Target methods
  void b_transport(tlm::tlm_generic_payload &payload,
                   sc_core::sc_time &delay_time);
  unsigned int transport_dbg(tlm::tlm_generic_payload& pl);
  bool get_direct_mem_ptr(tlm::tlm_generic_payload &payload,
                          tlm::tlm_dmi &dmi_data);

  void b_transport_word(const tlm::tlm_command command,
                        const uint32_t address,
                        uint32_t &data,
                        const uint32_t be_mask);

  void set_frequency_hook(unsigned);

private:
  const unsigned int m_size; // byte count
  uint32_t* m_storage;
  bool m_debug;
  bool m_dmi_allowed;
};

#endif // _MEMORY_HPP_
