/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _TEMP_SENSOR_HPP_
#define _TEMP_SENSOR_HPP_

#include "basic.hpp"
#include "pwt_module.hpp"

class TempSensor: public sc_core::sc_module,
                  public PwtModule
{
public:

  basic::target_socket<TempSensor> target;
  sc_core::sc_out<bool> irq;

  SC_HAS_PROCESS(TempSensor);
  TempSensor(sc_core::sc_module_name name);

  tlm::tlm_response_status
  read(const uint32_t& addr, uint32_t& data);
  
  tlm::tlm_response_status
  write(const uint32_t& addr, const uint32_t& data);

  void set_temperature_hook(double temp);  // Called by P/T solver
  inline void set_position(unsigned position) {m_position = position;}
  inline unsigned get_position() const {return m_position;}

protected:
  void send_interrupt(); // SC_THREAD

  unsigned m_position;

  uint32_t m_current_temp; 
  uint32_t m_temp_high; 
  uint32_t m_temp_low; 
  uint32_t m_interrupt;

  sc_core::sc_event m_interrupt_event;
};

#endif // _TEMP_SENSOR_HPP_
