/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

/*!
  \file hal.h
  \brief Harwdare Abstraction Layer : implementation for MicroBlaze
  ISS.

  
*/
#ifndef HAL_H
#define HAL_H

#include <stdint.h>
#include "fsl.h"
#include "memory_map.h"
#include "uart_regmap.h"

static inline uint32_t read_mem(uint32_t addr) {
  return * (volatile uint32_t *) addr;
}
static inline void write_mem(uint32_t addr, uint32_t data) {
  *((volatile uint32_t *) addr) = data;
}

#define wait_for_irq()  __asm(".byte 0x50, 0x0, 0x0, 0x0")/* Fake "sleep" instruction. */
/*  OP_IC_I initialize micro instructions counter to 0 */

//#define purge_time() __asm(".byte 0x54, 0x0, 0x0, 0x0")	

#define purge_time_auto() do {						\
    write_mem(MBWRAPPER_BASEADDR +  (MBWRAPPER_SIZE * get_cpu_id()) +  MUST_PURGE, PURGE_AUTO); \
    __asm("nop");                                                       \
  } while (read_mem(MBWRAPPER_BASEADDR + PURGE_REMAINING) == 1)

#define purge_time_preemt() do {                                \
    write_mem(MBWRAPPER_BASEADDR + MUST_PURGE, PREEMTIBLE);	\
    __asm("nop");						\
  } while (read_mem(MBWRAPPER_BASEADDR + PURGE_REMAINING) == 1)
		
#define purge_time_atomic() do {                                \
    write_mem(MBWRAPPER_BASEADDR + MUST_PURGE, NON_PREEMTIBLE); \
  } while (0)

#define purge_time_fake() do {                          \
    write_mem(MBWRAPPER_BASEADDR + MUST_PURGE, 0);      \
  } while (0)

#define cancel_n_functional() do {                      \
    write_mem(MBWRAPPER_BASEADDR + MUST_CANCEL, 1);     \
  } while (0)

#define cancel_n_functional_fake() do {                 \
    write_mem(MBWRAPPER_BASEADDR + MUST_CANCEL, 0);     \
  } while (0)


#define enable_interrupts() __asm("msrset r0, 0x2");

//while(read_mem(MBWRAPPER_BASEADDR + PURGE_REMAINING));	\

/* OP_IC_C print counted instruction to 0 */
#define check_inst_count(line)  __asm(".byte 0x58, 0x0, 0x0,"line) 
#define cpu_relax()     (void)NULL

#undef putchar
#define putchar simple_putchar
static inline void simple_putchar(int c) {
  write_mem(UART_BASEADDR+UART_WRITE_CHAR, c);
}

#define printf simple_printf
static inline int simple_printf(char *s) {
  for (; *s; s++) {
    simple_putchar((int) *s);
  }
}

int get_cpu_id() {
  int d; 
  int status; 
  fsl_get(0, d, status);
  return d; 
}

int setjmp();
void longjmp();

#endif /* HAL_H */
