/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

/*!
  \file intc-driver.h
  \brief Mini-driver for the IntC component
*/

#ifndef _INTC_DRIVER_H_
#define _INTC_DRIVER_H_

#include "hal.h"
#include "intc_regmap.h"
#include "memory_map.h"
#include <stdint.h>

static inline void ack_it(int n) {
  write_mem(INTC_BASEADDR + INTC_IAR, (1 << n));	
}

static inline void disable_it(int n) {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier &= ~(1<<n);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

static inline int get_it_status() {
  return read_mem(INTC_BASEADDR + INTC_ISR);
}

static inline void enable_it(int n) {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier |= (1<<n);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

static inline void init_intc() {
  write_mem(INTC_BASEADDR + INTC_IER, 0xFFFFFFFF);
}

static inline void disable_vga_it() {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier &= ~(0x1);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

static inline void enable_vga_it() {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier |= (0x1);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

static inline void disable_sensor_it() {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier &= ~(0x4);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

static inline void enable_sensor_it() {
  uint32_t ier = read_mem(INTC_BASEADDR + INTC_IER);
  ier |= (0x4);
  write_mem(INTC_BASEADDR + INTC_IER, ier);
}

#endif // _INTC_DRIVER_H_
