/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include <limits.h>
#include <stdlib.h>
#include <stdio.h>

#include "memory_map.h"
#include "vga_regmap.h"
#include "timer_regmap.h"
#include "temp_sensor_regmap.h"
#include "power_ctrl_regmap.h"
#include "hwgol_regmap.h"
#include "intc_driver.h"
#include "bit.h"
#include "hal.h"

#define SW_VGA_WIDTH  (VGA_WIDTH /2)  // "/2" <= pour accélérer
#define SW_VGA_HEIGHT (VGA_HEIGHT /2) //         la simulation

int irq_received = 0;

void set_pixel(uint32_t base_addr, int x, int y, int value) {
  uint32_t addr = base_addr +
    ALIGN( (x / CHAR_BIT) + (y * (VGA_LINE / CHAR_BIT)) );
  int bit = 31 - x % (sizeof(uint32_t) * CHAR_BIT);

  uint32_t data = read_mem(addr);
  if (value) {
    SET_BIT(&data, bit);
  } else {
    CLEAR_BIT(&data, bit);
  }
  write_mem(addr, data);
}

void clr_screen(uint32_t base_addr) {
  int i;
  int nbytes = (VGA_LINE * VGA_HEIGHT) / CHAR_BIT;
  for (i = 0; i < nbytes; i += sizeof(uint32_t)) {
    write_mem(base_addr + i, 0);
  }
}

void blank_screen(uint32_t base_addr) {
  int i;
  int nbytes = (VGA_LINE * VGA_HEIGHT) / CHAR_BIT;
  for(i = 0; i < nbytes; i += sizeof(uint32_t)) {
    write_mem(base_addr + i, 0xFFFFFFFF);
  }
}

int refresh, n;
uint32_t new_img_addr;
uint32_t old_img_addr;

// poor man's printf.
void print_hex(int i) {
  if (i > 0xf) {
    print_hex(i >> 4);
  }
  int c = i & 0xF;
  if (c < 10) {
    c += '0';
  } else {
    c = c - 10 + 'a';
  }
  putchar(c);
}

void draw_gun(int x, int y) {
  /* Left square */
  set_pixel(old_img_addr, x +  0, y + 5, 1);
  set_pixel(old_img_addr, x +  0, y + 6, 1);
  set_pixel(old_img_addr, x +  1, y + 5, 1);
  set_pixel(old_img_addr, x +  1, y + 6, 1);

  set_pixel(old_img_addr, x + 11, y + 5, 1);
  set_pixel(old_img_addr, x + 11, y + 6, 1);
  set_pixel(old_img_addr, x + 11, y + 7, 1);
  set_pixel(old_img_addr, x + 12, y + 4, 1);
  set_pixel(old_img_addr, x + 12, y + 8, 1);
  set_pixel(old_img_addr, x + 13, y + 3, 1);
  set_pixel(old_img_addr, x + 13, y + 9, 1);
  set_pixel(old_img_addr, x + 14, y + 3, 1);
  set_pixel(old_img_addr, x + 14, y + 9, 1);

  set_pixel(old_img_addr, x + 15, y + 6, 1);
  set_pixel(old_img_addr, x + 16, y + 4, 1);
  set_pixel(old_img_addr, x + 16, y + 8, 1);
  set_pixel(old_img_addr, x + 17, y + 5, 1);
  set_pixel(old_img_addr, x + 17, y + 6, 1);
  set_pixel(old_img_addr, x + 17, y + 7, 1);
  set_pixel(old_img_addr, x + 18, y + 6, 1);

  set_pixel(old_img_addr, x + 21, y + 3, 1);
  set_pixel(old_img_addr, x + 21, y + 4, 1);
  set_pixel(old_img_addr, x + 21, y + 5, 1);
  set_pixel(old_img_addr, x + 22, y + 3, 1);
  set_pixel(old_img_addr, x + 22, y + 4, 1);
  set_pixel(old_img_addr, x + 22, y + 5, 1);
  set_pixel(old_img_addr, x + 23, y + 2, 1);
  set_pixel(old_img_addr, x + 23, y + 6, 1);

  set_pixel(old_img_addr, x + 25, y + 1, 1);
  set_pixel(old_img_addr, x + 25, y + 2, 1);
  set_pixel(old_img_addr, x + 25, y + 6, 1);
  set_pixel(old_img_addr, x + 25, y + 7, 1);

  /* Right square */
  set_pixel(old_img_addr, x + 35, y + 3, 1);
  set_pixel(old_img_addr, x + 35, y + 4, 1);
  set_pixel(old_img_addr, x + 36, y + 3, 1);
  set_pixel(old_img_addr, x + 36, y + 4, 1);
}

int mod10(int a) {
  while (a>=10) a -= 10;
  return a;
}

void print_dec(int i) {
  write_mem(UART_BASEADDR + UART_WRITE_DEC, i);
}

int __start() {
  /* init_intc(); */
  write_mem(INTC_BASEADDR + INTC_IER, BIT(TIMER_IRQ));
	
  // CPU0 initiate screen and wait event (normal loop)
  printf("-- boot complete -- \n");

  refresh = 0;
  n = 0;
  old_img_addr = SRAM_BASEADDR;
  new_img_addr = SRAM_BASEADDR + 0x80000;

  clr_screen(old_img_addr);
  clr_screen(new_img_addr);
	
  draw_gun(10, 50);

  /* glider */
  set_pixel(old_img_addr, 11, 10, 1);
  set_pixel(old_img_addr, 12, 11, 1);
  set_pixel(old_img_addr, 10, 12, 1);
  set_pixel(old_img_addr, 11, 12, 1);
  set_pixel(old_img_addr, 12, 12, 1);
  
  /* another glider */
  set_pixel(old_img_addr, 51, 10, 1);
  set_pixel(old_img_addr, 50, 11, 1);
  set_pixel(old_img_addr, 52, 12, 1);
  set_pixel(old_img_addr, 51, 12, 1);
  set_pixel(old_img_addr, 50, 12, 1);

  /* another glider in right screen side */
  set_pixel(old_img_addr, SW_VGA_WIDTH/2 + 51, 10, 1);
  set_pixel(old_img_addr, SW_VGA_WIDTH/2 + 50, 11, 1);
  set_pixel(old_img_addr, SW_VGA_WIDTH/2 + 52, 12, 1);
  set_pixel(old_img_addr, SW_VGA_WIDTH/2 + 51, 12, 1);
  set_pixel(old_img_addr, SW_VGA_WIDTH/2 + 50, 12, 1);
  
  set_pixel(old_img_addr, 1, 31, 1);
  set_pixel(old_img_addr, 2, 31, 1);
  set_pixel(old_img_addr, 3, 31, 1);

  set_pixel(old_img_addr, 1, 91, 1);
  set_pixel(old_img_addr, 2, 91, 1);
  set_pixel(old_img_addr, 3, 91, 1);

  set_pixel(old_img_addr, 91, 31, 1);
  set_pixel(old_img_addr, 92, 31, 1);
  set_pixel(old_img_addr, 93, 31, 1);

  set_pixel(old_img_addr, 91, 91, 1);
  set_pixel(old_img_addr, 92, 91, 1);
  set_pixel(old_img_addr, 93, 91, 1);

  int i;
  for (i = 0; i<SW_VGA_WIDTH; ++i) {
    set_pixel(old_img_addr, i, SW_VGA_HEIGHT, 1);
    set_pixel(new_img_addr, i, SW_VGA_HEIGHT, 1);
  }
  for (i = 0; i<SW_VGA_HEIGHT; ++i) {
    set_pixel(old_img_addr, SW_VGA_WIDTH, i, 1);
    set_pixel(new_img_addr, SW_VGA_WIDTH, i, 1);
  }
  set_pixel(old_img_addr, SW_VGA_WIDTH, SW_VGA_HEIGHT, 1);
  set_pixel(new_img_addr, SW_VGA_WIDTH, SW_VGA_HEIGHT, 1);

  /* start vga */
  write_mem(VGA_BASEADDR + VGA_CFG_OFFSET, old_img_addr);

  /* start timer 1 */
  /* write_mem(TIMER_BASEADDR + TIMER_1_TLR_OFFSET, 50000); // 1ms */
  /* write_mem(TIMER_BASEADDR + TIMER_1_CSR_OFFSET, TIMER_RELOAD); */
  /* write_mem(TIMER_BASEADDR + TIMER_1_CSR_OFFSET, TIMER_START); */

  while (1) {
    
    write_mem(HWGOL_BASEADDR+HWGOL_DATA_SRC_REG, old_img_addr);
    write_mem(HWGOL_BASEADDR+HWGOL_DATA_DST_REG, new_img_addr);
    write_mem(INTC_BASEADDR + INTC_IER, BIT(HWGOL_IRQ));
    printf("-- master CPU computes frame: #");
    print_dec(n);
    printf("\n");
    write_mem(HWGOL_BASEADDR+HWGOL_START_REG, 1);

    while (1)
      wait_for_irq();
    printf("-- UNREACHABLE!");
  }

  return 0;
}

void vga_isr() {
  if (refresh) {
    write_mem(VGA_BASEADDR + VGA_CFG_OFFSET, old_img_addr);
    write_mem(HWGOL_BASEADDR+HWGOL_DATA_SRC_REG, old_img_addr);
    write_mem(HWGOL_BASEADDR+HWGOL_DATA_DST_REG, new_img_addr);
    write_mem(INTC_BASEADDR + INTC_IER, BIT(HWGOL_IRQ));
    refresh = 0;
    printf("-- master CPU computes frame: #");
    print_dec(++n);
    printf("\n");
    write_mem(HWGOL_BASEADDR+HWGOL_START_REG, 1);
  }
}

void timer_0_isr() {
  printf("timer_0_isr\n");
}

void timer_1_isr() {
  printf("timer_1_isr\n");
}

void hwgol_isr() {
  printf("hwgol_isr\n");
  const uint32_t tmp = old_img_addr;
  old_img_addr = new_img_addr;
  new_img_addr = tmp;
  refresh = 1;
  write_mem(INTC_BASEADDR + INTC_IER, BIT(VGA_IRQ));
}

void __interrupt() {
  irq_received = 1;
  if (get_cpu_id() != 0)
    return;
  uint32_t status = get_it_status();

  /* check if vga requested interrupt */
  if (TEST_BIT(status,VGA_IRQ)) {
    /* call the service routine */
    vga_isr();
    /* clear interrupt */
    write_mem(VGA_BASEADDR + VGA_INT_OFFSET, 1);
    ack_it(VGA_IRQ);
  }

  if (TEST_BIT(status,TIMER_IRQ)) {
    /* check if timer 0 requested interrupt */
    uint32_t timer_0_csr =
      read_mem(TIMER_BASEADDR + TIMER_0_CSR_OFFSET);
    if (timer_0_csr & TIMER_INTERRUPT) {
      /* call the service routine */
      timer_0_isr();
      /* clear the timer interrupt */
      write_mem(TIMER_BASEADDR + TIMER_0_CSR_OFFSET, timer_0_csr);
    }
    
    uint32_t timer_1_csr =
      read_mem(TIMER_BASEADDR + TIMER_1_CSR_OFFSET);
    if (timer_1_csr & TIMER_INTERRUPT) {
      /* call the service routine */
      timer_1_isr();
      /* clear the timer interrupt */
      write_mem(TIMER_BASEADDR + TIMER_1_CSR_OFFSET, timer_1_csr);
    }
    ack_it(TIMER_IRQ);
  }

  if (TEST_BIT(status,SENSOR_IRQ)) {
    ack_it(SENSOR_IRQ);    
  }

  if (TEST_BIT(status,HWGOL_IRQ)) {
    /* call the service routine */
    hwgol_isr();
    /* clear the timer interrupt */
    ack_it(HWGOL_IRQ);
  }
}
