/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */


#include "memory_map.h"

#include <limits.h>
#include <stdlib.h>
#include <stdio.h>

/* Must come after stdio.h to be able to modify libc functions
 * (printf, rand) after they are defined */
#include "hal.h"
#define CHECK_MEM
#define CHECK_OP

int __start() {
  while(1) {
    union {
      uint32_t word;
      uint16_t halves[2];
      uint8_t bytes[4];
    } deadbeef;
    deadbeef.word = 0xDEADBEEF;

    uint32_t* new_deadbeef;
    new_deadbeef = malloc(4);

    printf("START \n");
    // testing umod/udiv/umulsi3
    uint32_t x = 26;
    uint32_t y = 3;


#ifdef CHECK_OP		
    printf("testing mod ==>");
    if (x%y == 2)
      printf("ok \n");
    else 
      printf("NON \n");
		
    printf("testing div ==>");
    if (x/y == 8)
      printf("ok \n");
    else 
      printf("NON \n");

    printf("testing mulsi ==>");
    if (x*y == 78)
      printf("ok \n");
    else 
      printf("NON \n");
		
#endif
#ifdef CHECK_MEM		
    printf("Testing memory access half/byte/word \n" );
    // check read byte
    printf("Read byte ==>");
    if (deadbeef.bytes[0] != 0xDE) printf("ERROR-1\n");
    if (deadbeef.bytes[1] != 0xAD) printf("ERROR-2\n");
    if (deadbeef.bytes[2] != 0xBE) printf("ERROR-3\n");
    if (deadbeef.bytes[3] != 0xEF) printf("ERROR-4\n");
    printf(" OK\n");

    // check read half
    printf("Read half ==>");
    if (deadbeef.halves[0] != 0xDEAD) printf("ERROR-5\n");
    if (deadbeef.halves[1] != 0xBEEF) printf("ERROR-6\n");
    printf(" OK\n");

    // check write half
    printf("Write half ==>");
    deadbeef.halves[0] = 0xBEEF;
    deadbeef.halves[1] = 0xDEAD;
    if (deadbeef.word != 0xBEEFDEAD) printf("ERROR-7\n");
    printf(" OK\n");

    // check write byte
    printf("Write byte ==> ");
    deadbeef.bytes[0] = 0xDE;
    deadbeef.bytes[1] = 0xAD;
    deadbeef.bytes[2] = 0xBE;
    deadbeef.bytes[3] = 0xEF;
    if (deadbeef.word != 0xDEADBEEF) printf("ERROR-8\n");
    printf(" OK\n");

    // memcpy test 
    printf("testing memcpy ==>");
    memcpy((void*) new_deadbeef, deadbeef.bytes, 4);	
    if (*new_deadbeef != 0xDEADBEEF)
      printf("memcpy ERROR \n");
    else		
      printf("memcpy ok \n");
		
    printf("Memset ==> ");
    memset(deadbeef.bytes, 0xff, 2);
    if (deadbeef.word != 0xFFFFBEEF) printf("ERROR-9\n");
    printf(" OK\n");

    printf("Memset ==> ");
    memset(&deadbeef.halves[1], 0x00, 1);
    if (deadbeef.word != 0xFFFF00EF) printf("ERROR-10\n");
    printf(" OK\n");
#endif

    printf("if no error message Everything passed \n");			

    while (1) {
      wait_for_irq();
    }
  }
}

void __interrupt() {/* empty */}
