/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#ifndef _MB_WRAPPER_HPP_
#define _MB_WRAPPER_HPP_

#include "basic.hpp"
#include "microblaze.hpp"
#include <tlm_utils/simple_target_socket.h>
#include "processor_if.hpp"
#include "host_memory.hpp"
#include "pwt_module.hpp"

/*
 * Wrapper for MicroBlaze ISS using the BASIC protocol.
 */
   struct MBWrapper: sc_core::sc_module, ProcessorIF, PwtModule
{
  basic::initiator_socket<MBWrapper> initiator;

  sc_core::sc_in<bool>               irq;

  tlm_utils::simple_target_socket<MBWrapper> debug_target;

  void run_iss(void);
  void process_irq(void);

  typedef soclib::common::Iss iss_t;

  soclib::common::MicroBlazeIss m_iss;

  SC_HAS_PROCESS(MBWrapper);
  MBWrapper(sc_core::sc_module_name name,
            unsigned id = 0);
  ~MBWrapper();

  // ProcessorIF interface implementation
  virtual tlm::tlm_target_socket<32, tlm::tlm_base_protocol_types>&
  get_debug_target();

  virtual void set_endianness(endian_t);
  virtual void set_pc(uint32_t new_pc);

  // Numeric identifier
  const unsigned m_id;

private:
  void set_frequency_hook(unsigned hertz);

  bool m_clock_gated;
  int m_irq_duration;

  size_t m_step_count;
  sc_core::sc_time m_local_offset;

  HostMemory m_dmi;

  sc_core::sc_event m_freq_event;
  sc_core::sc_event interrupt_event;

  inline uint32_t read_word_or_abort(uint32_t addr,
                                     const char * msg = "memory read failed!");
  uint32_t read_word_or_abort_noinline(uint32_t addr, const char * msg);

  uint32_t read_half_or_abort(uint32_t addr,
                              const char * msg = "memory half read failed!");
  uint32_t read_byte_or_abort(uint32_t addr,
                              const char * msg = "memory byte read failed!");

  inline void write_word_or_abort(uint32_t addr, uint32_t data,
                                  const char * msg = "memory write failed!");
  void write_word_or_abort_noinline(uint32_t addr, uint32_t data, const char * msg);
  void write_half_or_abort(uint32_t addr, uint32_t data,
                           const char * msg = "memory half write failed!");
  void write_byte_or_abort(uint32_t addr, uint32_t data,
                           const char * msg = "memory byte write failed!");

  // Target methods (only transport_dbg is allowed)
  unsigned int transport_dbg(tlm::tlm_generic_payload& payload);
  void b_transport(tlm::tlm_generic_payload&, sc_core::sc_time&);
  bool get_direct_mem_ptr(tlm::tlm_generic_payload&, tlm::tlm_dmi&);
};

#endif // _MB_WRAPPER_HPP_
