/*
 * Copyright (C) 2009, 2010, 2011, 2012, 2013  by CNRS and Grenoble-INP
 *
 * This file is part of LIBTLMPWT
 *
 * LIBTLMPWT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors: Matthieu Moy <matthieu.moy@imag.fr>, Grenoble-INP
 *          Tayeb Bouhadiba, CNRS
 *          Claude Helmstetter, CNRS
 *
 * This software has been partially supported by the French ANR project HELP
 *                                                         (ANR-09-SEGI-006)
 */

#include "host_memory.hpp"
#include "trace.hpp"

using namespace std;
using namespace sc_core;
using namespace tlm;

namespace {
  const char* name() {return "DMI";}
  PwtTargetModule *none = NULL;
}

HostMemory::DMI_Descriptor::DMI_Descriptor():
  m_memory(NULL),
  m_size(0),
  m_offset(0),
  m_end(0),
  m_read_latency(SC_ZERO_TIME),
  m_write_latency(SC_ZERO_TIME),
  m_side_effects(&none)
{}

HostMemory::DMI_Descriptor::DMI_Descriptor(uint32_t *memory,
                                           uint32_t size,
                                           uint32_t offset,
                                           sc_time read_latency,
                                           sc_time write_latency,
                                           const vector<PwtTargetModule*> &side_effects):
  m_memory(memory),
  m_size(size),
  m_offset(offset),
  m_end(size+offset),
  m_read_latency(read_latency),
  m_write_latency(write_latency),
  m_side_effects(new PwtTargetModule*[side_effects.size()+1])
{
  std::copy(side_effects.begin(), side_effects.end(), m_side_effects);
  m_side_effects[side_effects.size()] = NULL;
}

HostMemory::HostMemory():
  dmi_descriptors(),
  m_memory(NULL), m_offset(~0), m_end(0)
{}

void HostMemory::get_dmi(uint32_t addr,
                         tlm_fw_direct_mem_if<tlm_generic_payload> &port)
{
  tlm_generic_payload payload;
  payload.set_command(tlm::TLM_IGNORE_COMMAND);
  payload.set_response_status(tlm::TLM_INCOMPLETE_RESPONSE);
  payload.set_address(addr);
  payload.set_data_ptr(NULL);
  payload.set_byte_enable_ptr(NULL);
  payload.set_data_length(4);
  payload.set_byte_enable_length(4);
  payload.set_streaming_width(4);
  vector<PwtTargetModule*> side_effects;
  side_effect_extension ext(&side_effects);
  payload.set_extension(&ext);
  tlm_dmi dmi_data; dmi_data.init();
  const bool status = port.get_direct_mem_ptr(payload, dmi_data);
  payload.clear_extension(&ext);
  if (!status || !dmi_data.is_read_write_allowed()) return;
  uint32_t *const ptr = reinterpret_cast<uint32_t*>(dmi_data.get_dmi_ptr());
  const uint32_t start = dmi_data.get_start_address();
  dmi_descriptors.push_back(DMI_Descriptor(ptr,
                                           dmi_data.get_end_address()+1-start, start,
                                           dmi_data.get_read_latency(),
                                           dmi_data.get_write_latency(),
                                           side_effects));
}

void HostMemory::clear() {
  INFO(2, "Clear DMI descriptors");
  dmi_descriptors.clear();
  m_memory = NULL;
  m_offset = m_end = 0;
  m_read_latency = SC_ZERO_TIME;
  m_write_latency = SC_ZERO_TIME;
  m_side_effects = &none;
}
